local class = require 'code/lib/middleclass'
local stateful = require 'code/lib/stateful'

Game = class('Game')
Game:include(stateful)

function Game:initialize(state)
    self:gotoState(state)
end



Menu = Game:addState('Menu')
Play = Game:addState('Play')
GameOver = Game:addState('GameOver')

-- Menu

function Menu:update(dt)
    -- start the game when the player presses a key
    if love.keyboard.isDown('space', 'return', 'up', 'down', 'left', 'right', 'w', 's', 'a', 'd') then
        self:gotoState('Play')
        return
    end
end

function Menu:draw()
    -- draw ground
    love.graphics.setColor(255, 255, 255)
    local w = images.background:getWidth()
    local h = images.background:getHeight()
    for x = 0, nativeCanvasWidth, w do
        for y = 0, nativeCanvasHeight, h do
            love.graphics.draw(images.background, x, y)
        end
    end

    -- draw title
    love.graphics.setFont(fonts.large)
    love.graphics.setColor(255, 250, 255)
    love.graphics.printf('Quinoa Vato', nativeCanvasWidth / 2 - 500, nativeCanvasHeight / 2 - 200, 1000, 'center')
    love.graphics.printf('Mission: Collect the apples to take home, avoid getting tacos!', nativeCanvasWidth / 2 - 480, nativeCanvasHeight / 2 - 100, 1000, 'center')
    -- press start
    if math.cos(2 * math.pi * love.timer.getTime()) > 0 then
        love.graphics.setFont(fonts.large)
        love.graphics.setColor(255, 255, 255)
        love.graphics.printf('PRESS ANY BUTTON TO START', nativeCanvasWidth / 2 - 500, nativeCanvasHeight / 2 + 50, 1000, 'center')
    end
end

-- Play

function Play:enteredState()
    -- music
    music.main:play()

    self.player = Player:new()
    self.director = Director:new()
    self.goodFoods = Container:new()
    self.badFoods = Container:new()
    self.goodFoods2 = Container:new()
    self.badFoods1 = Container:new()
    -- score
    self.score = 0

    -- timer
    self.timeLeft = 45
    
end

function Play:update(dt)
    self.player:update(dt)
    self.director:update(dt)
    self.goodFoods:update(dt)
    self.badFoods:update(dt)
    self.goodFoods2:update(dt)
    self.badFoods1:update(dt)

    -- update timer
    self.timeLeft = self.timeLeft - dt
    
        if self.timeLeft < 0 then
        self:gotoState("GameOver")
        return
    end

    -- back to menu
    if love.keyboard.isDown('escape') then
        self:gotoState('Menu')
        return
    end
end

function Play:draw()
    -- draw ground
    love.graphics.setColor(255, 255, 255)
    local w = images.background:getWidth()
    local h = images.background:getHeight()
    for x = 0, nativeCanvasWidth, w do
        for y = 0, nativeCanvasHeight, h do
            love.graphics.draw(images.background, x, y)
        end
    end

    self.goodFoods:draw()
    self.badFoods:draw()
    self.goodFoods2:draw()
    self.badFoods1:draw()
    self.player:draw()

    -- print score
    love.graphics.setFont(fonts.large)
    love.graphics.setColor(255, 255, 255)
    love.graphics.printf(self.score, nativeCanvasWidth - 1000 - 20, 0, 1000, 'right')

    -- print timer
    love.graphics.setFont(fonts.large)
    love.graphics.setColor(255, 255, 255)
    local seconds = round(self.timeLeft)
    if seconds < 10 then
        if math.cos(self.timeLeft * 12) > 0 then
            love.graphics.printf('0:0' .. seconds, 20, 0, 1000, 'left')
        end
    else
        love.graphics.printf('0:' .. seconds, 20, 0, 1000, 'left')
    end
end

function Play:exitedState()
    -- music
    music.main:stop()
end

-- GameOver

function GameOver:enteredState()
    -- sound effect
    sounds.bawselvl:play()

    -- timer
    self.initTime = love.timer.getTime()
end

function GameOver:update(dt)
    -- go back to menu after 10 seconds
    if love.timer.getTime() - self.initTime > 10 then
        self:gotoState('Menu')
        return
    end

    -- back to menu
    if love.keyboard.isDown('escape') then
        self:gotoState('Menu')
        return
    end
end

function GameOver:draw()
    -- draw ground
    love.graphics.setColor(255, 255, 255)
    local w = images.background:getWidth()
    local h = images.background:getHeight()
    for x = 0, nativeCanvasWidth, w do
        for y = 0, nativeCanvasHeight, h do
            love.graphics.draw(images.background, x, y)
        end
    end

    -- print score
    love.graphics.setFont(fonts.large)
    love.graphics.setColor(255, 255, 255)
    love.graphics.printf('Well done! You scored ' .. self.score .. '!', nativeCanvasWidth / 2 - 500, nativeCanvasHeight / 2 - 100, 1000, 'center')
end

function GameOver:exitedState()
    -- sound effect
    sounds.bawselvl:stop()
end